/**
 *  Elysion Framework for Processing.js
 *
 * (C) Johannes Stein, 2010
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Contact:
 * http://www.freeze-dev.de
 * johannesstein@freeze-dev.de
 *
 */

// Keeps track of how many objects have been created
int objectCount = 0;

// Define Content Root
String contentRoot = "";


/**
 *	Elysion Base Object
 *	Inherited from all other class in the Elysion Framework
 */ 
class elObject
{
	private
		String fUniqueID; // See: getUniqueID()
		String fType; // See: getType()

		/**
		  * Refreshes unique ID of an object
		  */
		void refreshUniqueID()
		{
			fUniqueID = fType + "-" + nf(int(random(1000)), 4) + "-" + nf(objectCount, 4);
		}
		
	public
		/**
		  * Contructor for base object
		  *
		  *	@constructor
		  */
		elObject()
		{
			objectCount++;
			fType = "elObject";
			refreshUniqueID();
		}
		
		/**
		  * The unique id of an object is specified as
		  * Object type - 4 digit random number - Object created
		  *
		  *	@returns Unique ID of an object as String
		  */
		String getUniqueID()
		{
			return fUniqueID;
		}
		
		/**
		  * The type of an object like elObject, elNode, etc.
		  * 
		  * @returns Type of the object as String
		  */
		String getType()
		{
			return fType;
		}
		
		/**
		  * Logs a specified message to the Processing.js debug console
		  */
		void log(String aMessage)
		{
			println(getUniqueID() + ": " + aMessage);
		}
} 


/************ BASIC DATA TYPES ************/
 
/**
  * Integer vector data type
  */
class elVector2i extends elObject
{
	public
		int x;
		int y;
		
		/**
		  * Constructor for elVector2i
		  *
		  * @constructor
		  */
		elVector2i()
		{
			super();
			
			fType = "elVector2i";
			refreshUniqueID();
			
			make(0, 0);
		}
		
		/**
		  * Alternative constructor for elVector2i
		  * Specify x- and y-position as parameter
		  *
		  * @constructor
		  */
		elVector2i(int aX, int aY)
		{
			super();
			
			fType = "elVector2i";
			refreshUniqueID();
		
			make(aX, aY);
		}
		
		/**
		  * Change x- and y-part in just one line instead of two :)
		  */
		void make(int aX, int aY)
		{
			this.x = aX;
			this.y = aY;	
		}
		
		/**
		  * Convert vector coordinates to String
		  *
		  * @returns vector coordinates as String
		  */
		String toString()
		{
			return "X: " + str(this.x) + " Y: " + str(this.y);
		}
		
		/**
		  * Adds another vector to this vector
		  */
		void add(elVector2i aVector)
		{
			this.x += aVector.x;
			this.y += aVector.y;
		}
		
		/**
		  * Substracts another vector to this vector
		  */
		void sub(elVector2i aVector)
		{
			this.x -= aVector.x;
			this.y -= aVector.y;
		}
		
		/**
		  * Multiplies another vector to this vector
		  */
		void multiply(elVector2i aVector)
		{
			this.x *= aVector.x;
			this.y *= aVector.y;
		}
		
		/**
		  * Divides another vector to this vector
		  */
		void divide(elVector2i aVector)
		{
			this.x /= aVector.x;
			this.y /= aVector.y;
		}
		
		/**
		  * Scales vector
		  */
		void scale(float aFactor)
		{
			this.x = int(this.x * aFactor);
			this.y = int(this.y * aFactor);
		}
		
		/**
		  * Get length of this vector
		  *
		  * @returns Length of this vector as float
		  */
		float getLength()
		{
			return sqrt(this.x * this.x + this.y * this.y);	
		}
		
		/**
		  * Normalize this vector
		  */
		void normalize()
		{
			this.x = int(this.x / getLength());
			this.y = int(this.y / getLength());
		}
		
		/**
		  * Dot product against vector given as parameter
		  */
		float dotProduct(elVector2i aVector)
		{
			return ((this.x * aVector.x) + (this.y * aVector.y));
		}
		
		/**
		  * Converts integer vector to float vector
		  */
		elVector2f toVector2f()
		{
		 	return new elVector2f(float(this.x), float(this.y));
		}
}

/**
  * Float vector data type
  *
  *
  */
class elVector2f extends elObject
{
	public
		float x;
		float y;
		
		/**
		  * Constructor for elVector2f
		  *
		  * @constructor
		  */
		elVector2f()
		{
			super();
			
			fType = "elVector2f";
			refreshUniqueID();
			
			make(0.0, 0.0);
		}
		
		/**
		  * Alternative constructor for elVector2f
		  * Specify x- and y-position as parameter
		  *
		  * @constructor
		  */
		elVector2f(float aX, float aY)
		{
			super();
			
			fType = "elVector2f";
			refreshUniqueID();
		
			make(aX, aY);
		}
		
		/**
		  * Change x- and y-part in just one line instead of two :)
		  */
		void make(int aX, int aY)
		{
			this.x = aX;
			this.y = aY;	
		}
		
		/**
		  * Convert vector coordinates to String
		  *
		  * @returns vector coordinates as String
		  */
		String toString()
		{
			return "X: " + str(this.x) + " Y: " + str(this.y);
		}
		
		/**
		  * Adds another vector to this vector
		  */
		void add(elVector2f aVector)
		{
			this.x += aVector.x;
			this.y += aVector.y;
		}
		
		/**
		  * Substracts another vector to this vector
		  */
		void sub(elVector2f aVector)
		{
			this.x -= aVector.x;
			this.y -= aVector.y;
		}
		
		/**
		  * Multiplies another vector to this vector
		  */
		void multiply(elVector2f aVector)
		{
			this.x *= aVector.x;
			this.y *= aVector.y;
		}
		
		/**
		  * Divides another vector to this vector
		  */
		void divide(elVector2f aVector)
		{
			this.x /= aVector.x;
			this.y /= aVector.y;
		}
		
		/**
		  * Scales vector
		  */
		void scale(float aFactor)
		{
			this.x *= aFactor;
			this.y *= aFactor;
		}
		
		/**
		  * Get length of this vector
		  *
		  * @returns Length of this vector as float
		  */
		float getLength()
		{
			return sqrt(this.x * this.x + this.y * this.y);	
		}
		
		/**
		  * Normalize this vector
		  */
		void normalize()
		{
			this.x = this.x / getLength();
			this.y = this.y / getLength();
		}
		
		/**
		  * Dot product against vector given as parameter
		  */
		float dotProduct(elVector2f aVector)
		{
			return ((this.x * aVector.x) + (this.y * aVector.y));
		}
		
		/**
		  * Converts float vector to integer vector
		  */
		elVector2i toVector2i()
		{
			return new elVector2i(int(this.x), int(this.y));	
		}
}

/**
  * elColor data type
  * Handles R, G, B and alpha value
  *
  */
class elColor extends elObject
{
	public
		byte r;
		byte g;
		byte b;
		byte a;
		
		/**
		  * Constructor for elColor
		  *
		  * @constructor
		  */
		elColor()
		{
			super();
			
			fType = "elColor";
			refreshUniqueID();
			
			make(255, 255, 255, 255);
		}
		
		/**
		  * Alternative constructor for elColor
		  * Specify R, G and B color value as parameter
		  *
		  * @constructor
		  */
		elColor(byte aR, byte aG, byte aB)
		{
			super();
			
			fType = "elColor";
			refreshUniqueID();
			
			make(aR, aG, aB);
		}
		
		/**
		  * Alternative constructor for elColor
		  * Specify R, G, B and alpha color value as parameter
		  *
		  * @constructor
		  */
		elColor(byte aR, byte aG, byte aB, byte anA)
		{
			super();
			
			fType = "elColor";
			refreshUniqueID();
			
			make(aR, aG, aB, anA);
		}
		
		void make(byte aR, byte aG, byte aB)
		{
			r = aR;
			g = aG;
			b = aB;
			a = 255;
		}
		
		void make(byte aR, byte aG, byte aB, byte anA)
		{
			r = aR;
			g = aG;
			b = aB;
			a = anA;
		}
		
		/**
		  * Convert color values to String
		  *
		  * @returns color values as String
		  */
		String toString()
		{
			return "R: " + str(this.r) + " G: " + str(this.g) + " B: " + str(this.b) + " A: " + str(this.a);
		}
		
		/**
		  * Adds another color to this color
		  */
		void add(elColor aColor)
		{
			this.r += aColor.r;
			this.g += aColor.g;
			this.b += aColor.b;
			this.a += aColor.a;
		}
		
		/**
		  * Substracts another color to this color
		  */
		void sub(elColor aColor)
		{
			this.r -= aColor.r;
			this.g -= aColor.g;
			this.b -= aColor.b;
			this.a -= aColor.a;
		}
		
		/**
		  * Multiplies another color to this color
		  */
		void multiply(elColor aColor)
		{
			this.r *= aColor.r;
			this.g *= aColor.g;
			this.b *= aColor.b;
			this.a *= aColor.a;
		}
		
		/**
		  * Divides another color to this color
		  */
		void divide(elColor aColor)
		{
			this.r /= aColor.r;
			this.g /= aColor.g;
			this.b /= aColor.b;
			this.a /= aColor.a;
		}
		
		/**
		  * Scales color
		  */
		void scale(float aFactor)
		{
			this.r *= aFactor;
			this.g *= aFactor;
			this.b *= aFactor;
			this.a *= aFactor;
		}
		
}

/**
  * elRect data type
  *
  */
class elRect extends elObject
{
	public
		int x;
		int y;
		int w;
		int h;
	
		/**
		  * Constructor for elRect
		  *
		  * @constructor
		  */
		elRect()
		{
			super();
			
			fType = "elRect";
			refreshUniqueID();
		
			make(0, 0, 0, 0);
		}
		
		/**
		  * Alternative constructor for elRect
		  * Specify x, y, width and height as parameter
		  *
		  * @constructor
		  */
		elRect(int aX, int aY, int aW, int aH)
		{
			super();
			
			fType = "elRect";
			refreshUniqueID();
		
			make(aX, aY, aW, aH);
		}
		
		void make(int aX, int aY, int aW, int aH)
		{
			x = aX;
			y = aY;
			w = aW;
			h = aH;
		}
		
		/**
		  * Convert rect int values to String
		  *
		  * @returns Rect values as String
		  */
		String toString()
		{
			return "X: " + str(this.x) + " Y: " + str(this.y) + " W: " + str(this.w) + " H: " + str(this.h);
		}
		
		/**
		  * Adds another color to this color
		  */
		void add(elRect aRect)
		{
			this.x += aRect.x;
			this.y += aRect.y;
			this.w += aRect.w;
			this.h += aRect.h;
		}
		
		/**
		  * Substracts another color to this color
		  */
		void sub(elRect aRect)
		{
			this.x -= aRect.x;
			this.y -= aRect.y;
			this.w -= aRect.w;
			this.h -= aRect.h;
		}
		
		/**
		  * Multiplies another color to this color
		  */
		void multiply(elRect aRect)
		{
			this.x *= aRect.x;
			this.y *= aRect.y;
			this.w *= aRect.w;
			this.h *= aRect.h;
		}
		
		/**
		  * Divides another color to this color
		  */
		void divide(elRect aRect)
		{
			this.x /= aRect.x;
			this.y /= aRect.y;
			this.w /= aRect.w;
			this.h /= aRect.h;
		}
		
		/**
		  * Scales color
		  */
		void scale(float aFactor)
		{
			this.x *= aFactor;
			this.y *= aFactor;
			this.w *= aFactor;
			this.h *= aFactor;
		}
		
		/**
		  * Checks if 
		  *
		  * @returns True 
		  */
		boolean contains(elVector2i aVector)
		{
			if ((aVector.x >= this.x) &&
				(aVector.y >= this.y) &&
				(aVector.x <= (this.x + this.w)) &&
				(aVector.y <= (this.y + this.h))) return true;
			else return false;				
		}
		
		boolean contains(elVector2f aVector)
		{
			return contains(aVector.toVector2i());
		}
		
		boolean contains(elRect aRect)
		{
			if ((aRect.x >= this.x) &&
				(aRect.y >= this.y) &&
				((aRect.x + aRect.w) <= (this.x + this.w)) &&
				((aRect.y + aRect.h) <= (this.y + this.h))) return true;
			else return false;
		}
}

/************ INPUT HELPER ************/

/**
  * KeyboardHelper class
  * Do not instance -> See: Input.Keyboard
  */
class elKeyboardHelper extends elObject
{
	public
		/**
		  * Constructor of elKeyboardHelper
		  *
		  * @constructor
		  */
		elKeyboardHelper()
		{
			super();
			
			fType = "elKeyboardHelper";
			refreshUniqueID();
		}
		
		/**
		  * Checks if Key is pressed down
		  *
		  * @returns True, if Key is pressed
		  */
		boolean isKeyDown()
		{
			
			return false;
		}
		
		/**
		  * Checks if Key has been pressed once
		  *
		  */
		boolean isKeyHit()
		{
			
			return false;
		}
		
		/**
		  * Checks if key has been released
		  *
		  */
		boolean isKeyUp()
		{
			
			return false;
		}
		
		/**
		  * Checks if any key has been pressed
		  *
		  * @returns True if any key has been pressed
		  */
		boolean anyKey()
		{
		
			return false;
		}
}

boolean mDragged = false;
boolean mPressed = false;
boolean mReleased = false;

void mouseDragged()
{
	mPressed = false;
	mDragged = true;
	mReleased = false;
}

void mousePressed()
{
	mPressed = true;
	mDragged = false;
	mReleased = false;
}

void mouseReleased()
{
	mPressed = false;
	mDragged = false;
	mReleased = true;	
}

/**
  * elMouseHelper
  *
  * Handles mouse click and cursor position
  */
class elMouseHelper extends elObject
{
	public
		/**
		  * Constructor for elMouseHelper
		  *
		  * @constructor
		  */
		elMouseHelper()
		{
			super();
			
			fType = "elMouseHelper";
			refreshUniqueID();
		}
	
		/**
		  *
		  *
		  */
		elVector2i cursor()
		{
			return new elVector2i(cursorX, cursorY);
		}
	
		boolean leftClick()
		{
			if (mousePressed && (mouseButton == LEFT))
	        {
	          mousePressed = false;
	          return true;
	        }

			return false;
		}
		
		boolean rightClick()
		{
			if (mousePressed && (mouseButton == RIGHT))
	        {
	          mousePressed = false;
	          return true;
	        }

			return false;
		}
		
		boolean isButtonDown()
		{
			return mPressed;
		}
		
		boolean isButtonHit()
		{
			return false;
		}
		
		boolean isButtonUp()
		{
			return mReleased;
		}
		
		boolean isDragging()
		{
			return mDragged;	
		}
}

/**
  * elInputHelper object
  *
  * Handles keyboard and mouse
  */
class elInputHelper extends elObject
{
	public
		elKeyboardHelper Keyboard;
		elMouseHelper Mouse;
		
		/**
		  * Constructor for elInputHelper
		  *
		  * @constructor
		  */
		elInputHelper()
		{
			super();
			
			fType = "elInputHelper";
			refreshUniqueID();
		
			Keyboard = new elKeyboardHelper();
			Mouse = new elMouseHelper();
		}
}

class elImageOffset
{
	public
		elVector2i position;
		elVector2i rotation;
		
		elImageOffset()
		{
			position = new elVector2i();
			rotation = new elVector2i();
		}
}

/**
  *	A node is a basic drawable object and is inherited by
  * every class that is drawing on the canvas
  *
  */
class elNode extends elObject
{

	public
		elVector2f position;
		elVector2f scale;
		elColor color;
		elImageOffset offset;
		float rotation;

        boolean visible;
	
		/**
		  * Constructor for node object
		  *
		  * @constructor
		  */
		elNode()
		{
			super();
			
			fType = "elNode";
			refreshUniqueID();
		
			position = new elVector2f();
			scale = new elVector2f();
			color = new elColor();
			offset = new elImageOffset();

            visible = true;
		}
		
		void move(elVector2i deltaVector)
		{
			move(deltaVector.toVector2f());
		}
		
		void move(elVector2f deltaVector)
		{
			position.add(deltaVector);	
		}
		
		/**
		  * Empty draw method
		  * Do not use directly
		  * Inherit from elNode and implement the draw method yourself
		  */
		void draw() {}
		
		/**
		  * Empty update method
		  * Do not use directly
		  * Inherit from elNode and implement the draw method yourself
		  */
		void update() {}
		
		/**
		  * Empty onMouseOver method
		  * Do not use directly
		  * Inherit from elNode and implement the draw method yourself
		  */
		boolean onMouseOver() 
		{
			return false;	
		}
		
		/**
		  * Empty onMouseOver method
		  * Do not use directly
		  * Inherit from elNode and implement the draw method yourself
		  */
		boolean onClick() 
		{
			return false;	
		}
}

/**
  *	elSprite Object
  * Loads an image file from a file and displays it on the screen
  *
  */
class elSprite extends elNode
{
	public
    	// Directly access the PImage if you feel like it
        PImage fImage;

		int width;
		int height;
		
		String errorString;
	
		/**
		  * Constructor for Sprite object
		  *
		  * @constructor
		  */
		elSprite()
		{
			super();
			
			fType = "elSprite";
			refreshUniqueID();
			
			fImage = new PImage();
			
			width = 0;
			height = 0;
			
			errorString = "";
			
			visible = true;
		}
	
		/**
		  * Loads a file from file
		  *
		  *
		  */
		boolean loadFromFile(String aFilename)
		{
			// Asychron image loading
			if (contentRoot != "") fImage = requestImage(contentRoot + "/" + aFilename);
    		else fImage = requestImage(aFilename);

            return isLoaded();
		}
		
		/**
		  * Since image loading is asynchron, we need a function if the image has been loaded yet
		  * If there was an error, an error string is written into errorString
		  *
		  * @returns True if image has been loaded, False if not
		  */
		boolean isLoaded()
		{
			if (fImage.width == 0) 
			{
				// Image is not loaded yet
				return false;
			} 
			else if (fImage.width == -1) 
			{
				errorString = "Error while loading image";
			} 
			else 
			{
				// Image is loaded, assing width and height values
			
				this.width = fImage.width;
				this.height = fImage.height;
				
				return true;
			}
		}
		
		/**
		  * Displays a sprite on the canvas
		  * Only displays an image if visible and is completely loaded
		  *
		  */
		void draw()
		{
			if (isLoaded())
			{
				if (visible)
				{
					pushMatrix();
					translate(position.x, position.y);
					rotate(radians(rotation));
					translate(-offset.position.x, -offset.position.y);
					image(fImage, -offset.rotation.x, -offset.rotation.y, (this.width * this.scale.x), (this.height * this.scale.y));
					translate(offset.position.x, offset.position.y);

					popMatrix();
				}
			}
		}

        boolean onMouseOver()
        {
        	if (isLoaded())
            {
            	if ((mouseX >= this.position.x) &&
                    (mouseY >= this.position.y) &&
            		(mouseX <= (this.position.x + (this.img.width * this.scale.x))) &&
            		(mouseY <= (this.position.y + (this.img.height * this.scale.y))))
                  return true;
                 else
                  return false;
            }
        }

        boolean onPoint(elVector2i aPoint)
        {
        	if (isLoaded())
            {
            	if ((aPoint.x >= this.position.x) &&
                    (aPoint.y >= this.position.y) &&
            		(aPoint.x <= (this.position.x + (this.img.width * this.scale.x))) &&
            		(aPoint.y <= (this.position.y + (this.img.height * this.scale.y))))
                  return true;
                 else
                  return false;
            }
        }

        boolean onPoint(elVector2f aPoint)
        {
        	if (isLoaded())
            {
            	if ((aPoint.x >= this.position.x) &&
                    (aPoint.y >= this.position.y) &&
            		(aPoint.x <= (this.position.x + (this.img.width * this.scale.x))) &&
            		(aPoint.y <= (this.position.y + (this.img.height * this.scale.y))))
                  return true;
                 else
                  return false;
            }
        }

        boolean onClick()
        {
        	if (this.onMouseOver())
            {
                if (mousePressed && (mouseButton == LEFT))
                {
                  mousePressed = false;
                  return true;
                }
             }

             return false;
        }
}

/**
  * elSpriteList Object
  *
  * So if you want to collect sprites, this is the way to do this :)
  */
class elSpriteList extends elObject
{
	private
		ArrayList fList;
	public
		/**
		  * SpriteList constructor
		  *
		  * @constructor
		  */
		elSpriteList()
		{
			super();
			
			fType = "elSpriteList";
			refreshUniqueID();
			
			fList = new ArrayList();
		}
		
		/**
		  * Add Sprite to the List
		  *
		  *
		  */
		void add(elSprite aSprite)
		{
			fList.add(aSprite);
		}
		
		void remove(int index)
		{
			fList.remove(index);
		}
		
		/**
		  * Draw method
		  * Draws all sprites from the SpriteList
		  */
		void draw()
		{
			for (int i = 0; i < fList.size() - 1; i++)
			{
				elSprite sprite = (elSprite) fList.get(i);
				sprite.draw();
			}
		}
}

/**
  * Loads a font from file and provides the possibility to
  * write some static text
  *
  */
class elFont extends elNode
{
	public
    	PFont font;
    	int fontSize;
        elColor fontColor;

    	elFont()
        {
        	super();

            fType = "elFont";
            refreshUniqueID();

            fontSize = 10;
            fontColor = new elColor();
        }

        void loadFromFile(String aFilename)
        {
            if (contentRoot != "") font = loadFont(contentRoot + "/" + aFilename);
            else font = loadFont(aFilename);
            textFont(font, fontSize);
        }

        /*void loadFromFile(String aFilename, int aFontSize)
        {
        	fontSize = aFontSize;
            loadFromFile(aFilename);
        }*/

        int getWidth(String aText)
        {
          return (font.width(aText) * fontSize);
        }

        int getHeight()
        {
          return fontSize;
        }

        void textOut(elVector2i aPosition, String aText)
        {
        	if (font)
            {
              fill(fontColor.r, fontColor.g, fontColor.b, fontColor.a);
              text(aText, aPosition.x, aPosition.y);
            }
        }

        void textOut(elVector2f aPosition, String aText)
        {
        	if (font)
            {
              fill(fontColor.r, fontColor.g, fontColor.b, fontColor.a);
              text(aText, aPosition.x, aPosition.y);
            }
        }
}

class elLabel extends elNode
{
	public
    	elFont font;
        String caption;

        elLabel()
		{
			super();

			fType = "elLabel";
			refreshUniqueID();

            caption = "";
		}

		void loadFromFile(String aFilename)
		{
        	font = new elFont();
            font.loadFromFile(aFilename);
		}

        void loadFromFont(elFont aFont)
        {
        	font = aFont;
        }

        int getWidth()
        {
        	return labelFont.getWidth(caption);
        }

        int getHeight()
        {
        	return labelFont.getHeight();
        }

        void draw()
        {
        	if (font)
            {
            	font.textOut(position, caption);
            }
        }

        void onMouseOver()
        {
          	if ((mouseX >= this.position.x) &&
                  (mouseY >= this.position.y) &&
          		(mouseX <= (this.position.x + (this.labelFont.getWidth(caption)))) &&
          		(mouseY <= (this.position.y + (this.labelFont.getHeight()))))
                return true;
               else
                return false;
        }

        boolean onClick()
        {
        	if (this.onMouseOver())
            {
                if (mousePressed && (mouseButton == LEFT))
                {
                  mousePressed = false;
                  return true;
                }
             }

             return false;
    	}
}

/**
  *
  *
  *
  */
class elButton extends elNode
{
	public
		elSprite sprite;
        elLabel label;

		elButton()
		{
			super();

			fType = "elButton";
			refreshUniqueID();

            sprite = new elSprite();
            label = new elLabel();
		}

		void loadFromFile(String aFilename)
        {
        	sprite.loadFromFile(aFilename);
        }

        void loadFont(String fontName)
        {
        	label.loadFromFile(fontName);
        }

        void draw()
        {
        	sprite.position = this.position;
            label.position.x = sprite.position.x + (sprite.getWidth() - label.getWidth()) / 2;
            label.position.y = sprite.position.y + (sprite.getHeight() - label.getHeight()) / 2;
        }

        boolean onMouseOver()
        {
        	return sprite.onMouseOver();
        }

        boolean onClick()
        {
        	return sprite.onClick();
        }
}

class elGUISkin extends elObject
{
	elGUISkin()
	{
	
	}
}

class elGUI extends elObject
{
	elGUI()
	{
	
	}
	
	elGUI(elGUISkin guiSkin)
	{
	
	}
	
	void box(elRect aRect, elColor aColor)
	{
		stroke(aColor.r, aColor.g, aColor.b, aColor.a);
		fill(aColor.r, aColor.g, aColor.b, aColor.a);
		rect(aRect.x, aRect.y, aRect.w, aRect.h);
	}
	
	/*boolean Button()
	{
	
	}
	
	boolean Label()
	{
	
	}*/
}

class elLayer extends elNode
{
	public
		elLayer()
		{
			super();
		}
}

class elScene extends elObject
{
	private
		ArrayList nodelist;
		ArrayList nodetypes;
	public
		elScene()
		{
			super();
		
			nodelist = new ArrayList();
			nodetypes = new ArrayList();
		}
		
		void add(elNode node)
		{
			nodelist.add(node);
		}
		
		void draw()
		{
			for (int i = 0; i < nodelist.size()-1; i++)
			{
		
			}
		}
		
		void update() { }
}

boolean onPoint(elRect aRect, elVector2i aPoint)
{
	if ((aPoint.x >= aRect.x) &&
        (aPoint.y >= aRect.y) &&
		(aPoint.x <= (aRect.x + aRect.w)) &&
		(aPoint.y <= (aRect.y + aRect.h)))
    return true;
   else
    return false;
}

// Init objects
elInputHelper Input = new elInputHelper();
elGUI GUI = new elGUI();
